@@ -1,26 +1,28 @@
+
 package Term::TablePrint;
 
 use warnings;
 use strict;
-use 5.010001;
+use 5.008000;
+no warnings 'utf8';
 
-our $VERSION = '0.010_01';
+our $VERSION = '0.018_01';
 use Exporter 'import';
 our @EXPORT_OK = qw( print_table );
 
-use Carp          qw( carp croak );
-use List::Util    qw( sum );
-use Scalar::Util  qw( looks_like_number );
+use Carp         qw( carp croak );
+use List::Util   qw( sum );
+use Scalar::Util qw( looks_like_number );
 
 use Term::Choose       qw( choose );
 use Term::Choose::Util qw( term_size insert_sep unicode_sprintf );
-use Term::ProgressBar;
-use Text::LineFold;
-use Unicode::GCString;
+use Term::ProgressBar  qw();
+use Text::LineFold     qw();
+use Unicode::GCString  qw();
+
+sub CLEAR_SCREEN () { "\e[H\e[J" }
 
-no warnings 'utf8';
 
-sub CLEAR_SCREEN () { "\e[1;1H\e[0J" }
 
 sub new {
     my $class = shift;
@@ -34,9 +36,10 @@ sub new {
     return $self;
 }
 
+
 sub __validate_options {
     my ( $self, $opt ) = @_;
-    if ( $opt->{db_browser_mode} ) {
+    if ( $opt->{_db_browser_mode} || $opt->{db_browser_mode} ) { ###
         %$self = ( %$self, %$opt );
         return;
     }
@@ -79,18 +82,18 @@ sub __validate_options {
 
 sub __set_defaults {
     my ( $self ) = @_;
-    $self->{progress_bar}   //= 20000;
-    $self->{max_rows}       //= 50000;
-    $self->{tab_width}      //= 2;
-    $self->{min_col_width}  //= 30;
-    $self->{table_expand}   //= 1;
-    $self->{binary_filter}  //= 0;
-    $self->{undef}          //= '';
-    $self->{mouse}          //= 0;
-    $self->{binary_string}  //= 'BNRY';
-    $self->{choose_columns} //= 0;
-    $self->{add_header}     //= 0;
-    $self->{keep_header}    //= 1;
+    $self->{progress_bar}   = 40000  if ! defined $self->{progress_bar};
+    $self->{max_rows}       = 50000  if ! defined $self->{max_rows};
+    $self->{tab_width}      = 2      if ! defined $self->{tab_width};
+    $self->{min_col_width}  = 30     if ! defined $self->{min_col_width};
+    $self->{table_expand}   = 1      if ! defined $self->{table_expand};
+    $self->{binary_filter}  = 0      if ! defined $self->{binary_filter};
+    $self->{undef}          = ''     if ! defined $self->{undef};
+    $self->{mouse}          = 0      if ! defined $self->{mouse};
+    $self->{binary_string}  = 'BNRY' if ! defined $self->{binary_string};
+    $self->{choose_columns} = 0      if ! defined $self->{choose_columns};
+    $self->{add_header}     = 0      if ! defined $self->{add_header};
+    $self->{keep_header}    = 1      if ! defined $self->{keep_header};
     $self->{thsd_sep} = ',';
     $self->{no_col}   = 'col';
 }
@@ -160,13 +163,14 @@ sub print_table {
     my $self = shift;
     my ( $table_ref, $opt ) = @_;
     my  $a_ref;
-    if ( $self->{db_browser_mode} ) {
+    if ( $self->{_db_browser_mode} || $self->{db_browser_mode} ) { ###
         $self->__set_defaults();
         $a_ref = $table_ref;
     }
     else {
         croak "print_table: called with " . @_ . " arguments - 1 or 2 arguments expected." if @_ < 1 || @_ > 2;
         croak "print_table: Required an ARRAY reference as the first argument."            if ref $table_ref  ne 'ARRAY';
+        croak "print_table: Empty table without header row!"                               if ! @$table_ref;
         if ( defined $opt ) {
             croak "print_table: The (optional) second argument is not a HASH reference."   if ref $opt ne 'HASH';
             $self->{backup_opt} = { map{ $_ => $self->{$_} } keys %$opt } if defined $opt;
@@ -198,12 +202,24 @@ sub print_table {
     my $gcs_bnry = Unicode::GCString->new( $self->{binary_string} );
     $self->{binary_length} = $gcs_bnry->columns;
     if ( $self->{progress_bar} ) {
-        say 'Computing: ...';
-        if ( @$a_ref * @{$a_ref->[0]} > $self->{progress_bar} ) {
-            $self->{show_progress} = 1;
-        }
+        print 'Computing: ...' . "\n";
+        $self->{show_progress} = int @$a_ref * @{$a_ref->[0]} / $self->{progress_bar};
     }
     $self->__calc_col_width( $a_ref );
+
+    #########################
+    if ( defined wantarray ) {
+        if ( wantarray ) {
+            my @idxs = $self->__inner_print_tbl( $a_ref );
+            return @idxs;
+        }
+        else {
+            my $idx = $self->__inner_print_tbl( $a_ref );
+            return $idx;
+        }
+    }
+    #########################
+
     $self->__inner_print_tbl( $a_ref );
     if ( $self->{backup_opt} ) {
         my $backup_opt = delete $self->{backup_opt};
@@ -220,16 +236,6 @@ sub __inner_print_tbl {
     my $width_cols = $self->__calc_avail_width( $a_ref, $term_width );
     return if ! $width_cols;
     my ( $list, $len ) = $self->__trunk_col_to_avail_width( $a_ref, $width_cols );
-
-    ####
-    if ( @$list == 1 && $self->{keep_header} ) {
-        choose(
-            [ "Empty Table!", map { " $_" } @{$a_ref->[0]} ],
-            { prompt => '', clear_screen => 1, layout => 3, mouse => $self->{mouse} }
-        );
-    }
-    ####
-
     if ( $self->{max_rows} && @$list - 1 >= $self->{max_rows} ) {
         my $reached_limit = 'REACHED LIMIT "MAX_ROWS": ' . insert_sep( $self->{max_rows}, $self->{thsd_sep} );
         my $gcs1 = Unicode::GCString->new( $reached_limit );
@@ -242,6 +248,29 @@ sub __inner_print_tbl {
         }
         push @$list, unicode_sprintf( $reached_limit, $len, 0 );
     }
+
+    #########################
+    if ( defined wantarray ) {
+        my $prompt = 'Your choice:';
+        #shift @$list; # The index 0 is the index of the row after the header row.
+        if ( wantarray ) {
+            my @idxs = choose(
+                $list,
+                { prompt => $prompt, index => 1, ll => $len, layout => 3, clear_screen => 1, mouse => $self->{mouse} }
+            );
+            return @idxs;
+        }
+        else {
+            my $idx = choose(
+                $list,
+                { prompt => $prompt, index => 1, ll => $len, layout => 3, clear_screen => 1, mouse => $self->{mouse} }
+            );
+            return $idx;
+        }
+    }
+    #########################
+
+    #my $old_row = $self->{keep_header} ? @$list : 0;
     my $old_row = 0;
     my $auto_jump = 1;
     my ( $width ) = term_size();
@@ -251,25 +280,12 @@ sub __inner_print_tbl {
             $self->__inner_print_tbl( $a_ref );
             return;
         }
-
-#        my $row = choose(
-#            $list,
-#            { prompt => '', index => 1, default => $old_row, ll => $len, layout => 3,
-#              clear_screen => 1, mouse => $self->{mouse} }
-#        );
-#        return if ! defined $row;
-#        return if $row == 0;
-#        if ( ! $self->{table_expand} ) {
-#            $old_row = 0;
-#            next;
-#        }
-#        if ( $old_row == $row ) {
-#            $old_row = 0;
-#            next;
-#        }
-#        $old_row = $row;
-        ####
         my $prompt = $self->{keep_header} ? shift @$list : '';
+        if ( ! @$list ) {
+            push @$list, $prompt;
+            $prompt = '';
+            $old_row = 0;
+        }
         my $row = choose(
             $list,
             { prompt => $prompt, index => 1, default => $old_row, ll => $len, layout => 3,
@@ -277,6 +293,15 @@ sub __inner_print_tbl {
         );
         unshift @$list, $prompt if $self->{keep_header};
         return if ! defined $row;
+        #if ( ! $self->{table_expand} ) {
+        #    return if $row == 0;
+        #    next;
+        #}
+        #if ( $old_row == $row ) {
+        #    return if $row == 0;
+        #    $old_row = 0;
+        #    next;
+        #}
         if ( ! $self->{table_expand} ) {
             return if $row == 0;
             next;
@@ -295,7 +320,6 @@ sub __inner_print_tbl {
         $auto_jump = 0;
         $old_row = $row;
         $row++ if $prompt;
-        ####
 
         my $row_data = $self->__single_row( $a_ref, $row, $self->{longest_col_name} + 1 );
         choose(
@@ -348,9 +372,23 @@ sub __calc_col_width {
     my $normal_row = 0;
     $self->{width_cols} = [ ( 1 ) x @{$a_ref->[0]} ];
     my @col_idx = ( 0 .. $#{$a_ref->[0]} );
+    my $show_progress = $self->{show_progress} >= 2 ? 1 : 0; #
+    my $total = $#{$a_ref};                   #
+    my $next_update = 0;                      #
+    my $c = 0;                                #
+    my $progress;                             #
+    if ( $show_progress ) {                   #
+        local $| = 1;                         #
+        print CLEAR_SCREEN;                   #
+        $progress = Term::ProgressBar->new( { #
+            name => 'Computing',              #
+            count => $total,                  #
+            remove => 1 } );                  #
+        $progress->minor( 0 );                #
+    }                                         #
     for my $row ( @$a_ref ) {
         for my $i ( @col_idx ) {
-            $row->[$i] //= $self->{undef};
+            $row->[$i] = $self->{undef} if ! defined $row->[$i];
             if ( ref $row->[$i] ) {
                 $row->[$i] = $self->__handle_reference( $row->[$i] );
             }
@@ -379,7 +417,16 @@ sub __calc_col_width {
                 $normal_row = 1 if $i == $#$row;
             }
         }
+        if ( $show_progress ) {                                              #
+            my $is_power = 0;                                                #
+            for ( my $i = 0; 2 ** $i <= $c; ++$i ) {                         #
+                $is_power = 1 if 2 ** $i == $c;                              #
+            }                                                                #
+            $next_update = $progress->update( $c ) if $c >= $next_update;    #
+            ++$c;                                                            #
+        }                                                                    #
     }
+    $progress->update( $total ) if $show_progress && $total >= $next_update; #
 }
 
 sub __handle_reference {
@@ -433,7 +480,7 @@ sub __calc_avail_width {
     elsif ( $sum > $avail_width ) {
         my $minimum_with = $self->{min_col_width} || 1;
         if ( @$width_head > $avail_width ) {
-            say 'Terminal window is not wide enough to print this table.';
+            print 'Terminal window is not wide enough to print this table.' . "\n";
             choose(
                 [ 'Press ENTER to show the column names.' ],
                 { prompt => '', clear_screen => 0, mouse => $self->{mouse} }
@@ -494,8 +541,6 @@ sub _minus_x_percent {
 }
 
 
-
-
 sub __trunk_col_to_avail_width {
     my ( $self, $a_ref, $width_cols ) = @_;
     my $total = $#{$a_ref};                   #
@@ -510,7 +555,7 @@ sub __trunk_col_to_avail_width {
             count => $total,                  #
             remove => 1 } );                  #
         $progress->minor( 0 );                #
-    }
+    }                                         #
     my $list;
     my $tab = ' ' x $self->{tab_width};
     for my $row ( @$a_ref ) {
@@ -551,7 +596,7 @@ Term::TablePrint - Print a table to the terminal and browse it interactively.
 
 =head1 VERSION
 
-Version 0.010_01
+Version 0.018_01
 
 =cut
 
@@ -583,15 +628,25 @@ If the table has more rows than the terminal, the table is divided up on as many
 cursor reaches the end of a page, the next page is shown automatically until the last page is reached. Also if the
 cursor reaches the topmost line, the previous page is shown automatically if it is not already the first one.
 
-If the terminal is too narrow to print the table, the columns are adjusted to the available width automatically. If the
-option I<table_expand> is enabled and a row is selected with C<Return>, each column of that row is output in its own
-line preceded by the column name. This might be useful if the columns were cut due to the too low terminal width.
+If the terminal is too narrow to print the table, the columns are adjusted to the available width automatically.
+
+=head3 print_table called in void context.
+
+If the option I<table_expand> is enabled and a row is selected with C<Return>, each column of that row is output in its
+own line preceded by the column name. This might be useful if the columns were cut due to the too low terminal width.
+
+=head3 print_table called in scalar context.
+
+If a row is selected with C<Return>, C<print_table> returns the index of that row.
+
+=head3 print_table called in list context.
+
+In list context it can be marked rows with the C<SpaceBar>. When C<Return> is pressed C<print_table> returns the indexes
+of the marked row including the highlighted row.
 
 To get a proper output C<print_table> uses the C<columns> method from L<Unicode::GCString> to calculate the string
 length.
 
-If STDOUT is redirected C<print_table> prints the table to STDERR.
-
 The following modifications are made (at a copy of the original data) before the output.
 
 Leading and trailing spaces are removed from the array elements
@@ -668,11 +723,10 @@ row of the table.
 
 =back
 
-The C<Return> key closes the table if the cursor is on the header row unless I<table_expand> and I<keep_header> are
-enabled. If I<table_expand> and I<keep_header> are enabled, the output closes if C<Return> is pressed twice in
-succession; this special behavior may change (or be removed) in a future release.
+The C<Return> key closes the table if the cursor is on the header row. If I<keep_header> and I<table_expand> are
+enabled, the table closes by selecting the first row twice in succession.
 
-If the cursor is not on the header row:
+If the cursor is not on the first row:
 
 =over
 
@@ -684,7 +738,8 @@ with the option I<table_expand> disabled the cursor jumps to the table head if C
 
 with the option I<table_expand> enabled each column of the selected row is output in its own line preceded by the
 column name if C<Return> is pressed. Another C<Return> closes this output and goes back to the table output. If a row is
-selected twice in succession, the pointer jumps to the head of the table.
+selected twice in succession, the pointer jumps to the head of the table or to the first row if I<keep_header> is
+enabled.
 
 =back
 
@@ -761,7 +816,7 @@ Default: 0
 Set the progress bar threshold. If the number of fields (rows x columns) is higher than the threshold, a progress bar is
 shown while preparing the data for the output.
 
-Default: 20_000
+Default: 40_000
 
 =head3 tab_width
 
@@ -774,7 +829,7 @@ Default: 2
 If the option I<table_expand> is set to 1 and C<Return> is pressed, the selected table row is printed with each column
 in its own line.
 
-If I<table_expand> is set to 0, the cursor jumps to the to header row (if not already there) when C<Return> is pressed.
+If I<table_expand> is set to 0, the cursor jumps to the to first row (if not already there) when C<Return> is pressed.
 
 Default: 1
 
@@ -818,11 +873,13 @@ if an invalid option value is passed.
 
 =back
 
+if the first argument refers to an empty array.
+
 =head1 REQUIREMENTS
 
 =head2 Perl version
 
-Requires Perl version 5.10.1 or greater.
+Requires Perl version 5.8.0 or greater.
 
 =head2 Decoded strings
 
